﻿using System;
using System.Collections.Generic;
using PX.SM;
using PX.Data;
using PX.Objects.CS;
using PX.Objects.IN;
using PX.Objects.CR;
using PX.Objects.AR;

namespace AcumaticaDeviceHub
{
    public class PEPrintSLMaint : PXGraph<PEPrintSLMaint>
    {
        #region DAC PrintParameters
        [System.SerializableAttribute]
        public partial class PrintParameters : IBqlTable, PX.SM.IPrintable
        {
            #region PrintWithDeviceHub
            public abstract class printWithDeviceHub : IBqlField
            {
            }
            [PXDBBool]
            [PXDefault(typeof(FeatureInstalled<FeaturesSet.deviceHub>))]
            [PXUIField(DisplayName = "Print with DeviceHub")]
            public virtual bool? PrintWithDeviceHub { get; set; }
            #endregion
            #region DefinePrinterManually
            public abstract class definePrinterManually : IBqlField
            {
            }
            [PXDBBool]
            [PXDefault(true)]
            [PXUIField(DisplayName = "Define Printer Manually")]
            public virtual bool? DefinePrinterManually { get; set; }
            #endregion
            #region Printer
            public abstract class printerName : PX.Data.IBqlField { }
            [PX.SM.PXPrinterSelector]
            public virtual string PrinterName { get; set; }
            #endregion
        }
        #endregion

        public static class Parameters
        {
            public const string Inventory_ID = "Inventory_ID";
        }

        public static class Fields
        {
            public static readonly string Inventory_ID = string.Format("{0}.{1}", nameof(InventoryItem), nameof(Parameters.Inventory_ID));
        }

        #region Methods
        public void PrintReportInDeviceHub(string reportID, string actualReportID, List<string> nameGroupsList, Dictionary<string, string> parametersDictionary, string printerName, int numberPrint)
        {
            Guid loggedUser = this.Accessinfo.UserID;

            Users usersRow = PXSelect<Users,
                                    Where<Users.pKID, Equal<Required<Users.pKID>>>>
                                    .Select(this, loggedUser);
            string BinaryGroupMask = Convert.ToString(usersRow.GroupMask[0], 2);

            List<string> ListGroupsUser = new List<string>();
            for (int i = 0; i < BinaryGroupMask.Length; i++)
            {
                if (BinaryGroupMask.Substring(i, 1) != "0")
                {
                    String BinaryGroup = (BinaryGroupMask.Substring(i, 1).PadLeft(i + 1, '0')).PadRight(BinaryGroupMask.Length, '0');
                    ListGroupsUser.Add(Convert.ToString(Convert.ToDecimal(Convert.ToByte(BinaryGroup, 2))));
                }

            }

            Dictionary<string, string> groupsUserDictionary = new Dictionary<string, string>();
            foreach (String GroupUser in ListGroupsUser)
            {
                foreach (RelationGroup relationGroupRow in PXSelect<RelationGroup, Where<RelationGroup.active, Equal<Required<RelationGroup.active>>>>.Select(this, 1))
                {
                    if (Convert.ToString(relationGroupRow.GroupMask[0]) == GroupUser)
                    {
                        groupsUserDictionary[relationGroupRow.GroupName] = Convert.ToString(relationGroupRow.GroupMask[0]);
                        break;
                    }
                }
            }

            foreach (SMPrinter printersRow in PXSelect<SMPrinter, Where<SMPrinter.isActive, Equal<Required<SMPrinter.isActive>>>>.Select(this, 1))
            {
                foreach (string nameGroup in nameGroupsList)
                {
                    if (groupsUserDictionary.ContainsKey(nameGroup))
                    {
                        if (groupsUserDictionary[nameGroup] == Convert.ToString(printersRow.GroupMask[0]))
                        {
                            printerName = printersRow.PrinterName;
                            break;
                        }
                    }
                }
            }

            if (String.IsNullOrEmpty(printerName))
            {
                throw new PXException(Convert.ToString("The user is not active in an impression group"));
            }

            Dictionary<string, PXReportRequiredException> reportsToPrint = new Dictionary<string, PXReportRequiredException>();
            PrintParameters filter = new PrintParameters();
            filter.PrintWithDeviceHub = true;
            filter.DefinePrinterManually = true;
            filter.PrinterName = printerName;

            for (int i = 0; i < numberPrint; i++)
            {
                reportsToPrint = PX.SM.SMPrintJobMaint.AssignPrintJobToPrinter(reportsToPrint, parametersDictionary, filter,
                new NotificationUtility(this).SearchPrinter, ARNotificationSource.Customer, reportID, actualReportID, 16);
            }

            if (reportsToPrint != null)
            {
                PX.SM.SMPrintJobMaint.CreatePrintJobGroups(reportsToPrint);
            }
        }
        #endregion
    }
}